%%
%   IBM 4758  CCA API
%
%   Recomendation 1, Role B:
%
%   The attacker has acces to Encipher, Decipher, KeyExport,
%   Symmetric-Key-Import and KeyTranslate commands.
%%

    fun enc/2.      % symmetric encryption
    fun penc/2.     % public key encryption

    % types of keys:
    % fun dat/0.    % data keys  (data = 0)
    fun imp/0.      % import keys
    fun exp/0.      % export keys
    fun pin/0.      % key derivation keys
    fun kp/0.       % partial key type

    fun km/0.       % the security module's master key 
    fun pan/0.      % an account number

    fun c/0.        % some value known by the intruder

    fun pk/0.       % private key of the ATM 
    fun pub/1.      % a public kay related to ...
    fun key/1.      % key(Type) -- a key of type Type generated by the module or guessed.
    fun kek/0.
    fun ukek1/0.   
    fun ukek2/0.   

    fun transf/2.   %

    fun pdkey/0.    % a Pin Derivation Key

    fun sec/0.

%%%%%%%%%%%% Auxiliary Prolog predicates

    type(0).   % data=0
    type(imp).
    type(exp).
    type(pin).
    
%%%%%%%%%%%% THE INITIAL KNOWLEDGE %%%%%%%%

    rule c.
    % rule dat.  % dat=0
    rule imp.
    rule exp.
    rule pin.
    rule kp.

    rule pan.

    % rule pub(pk).
    % rule enc(pdkey, km+pin).  
    rule penc((kek,imp), pk). 
    rule enc(pdkey, kek+pin). 

    %%% Key Generating/Guessing

    rule enc(key(0),   km+0).
    rule key(0).
    rule enc(key(imp), km+imp).
    rule enc(key(exp), km+exp).
    rule enc(key(pin), km+pin).

    rule enc(ukek1, km+kp+0).
    rule enc(ukek1, km+kp+imp).
    rule enc(ukek1, km+kp+exp).
    rule enc(ukek1, km+kp+pin).
    
    % For IBM Attack:
    rule enc(ukek2, ukek1+exp).

    % ... and some similar keys
    rule enc(ukek2, ukek1+imp).
    rule enc(ukek2, ukek1+pin).
    rule enc(ukek2, ukek1+exp).
 

%%%%%%%%%%%% INTRUDER RULES %%%%%%%%%%%%%%%

    rule  X, K -> enc(X,K).
    rule  enc(X,K), K -> X.

    rule  X, pub(K) -> penc(X,K).
    rule  penc(X,K), K -> X.
%%%%%%%%%%%% PROTOCOL RULES %%%%%%%%%%%%%%%

%%% Encipher
    rule  X, enc(Key, km+0)  ->  enc(X, Key).

%%% Decipher
    rule  enc(X, Key), enc(Key, km+0)  ->  X.

%%% Key Export
    rule  enc(Key, km+Type), Type, enc(KEK, km+exp)  ->  enc(Key, KEK+Type)
    :- type(Type).

%%% Symmetric Key Import

    rule penc( (K,Type), pk ) -> enc(K, km+Type)
    :- type(Type).

%%% Key Translate (not subsumed in this case):

    % rule  enc(Key, Kek1+Type), Type, enc(Kek1, km+imp), enc(Kek2, km+exp)  ->  enc(Key, Kek2+Type)
    % :- type(Type).
    %
    % Instead of the rule above, we have the two rules:
    %

    rule  enc(Key, Kek1+Type), Type, enc(Kek1, km+imp)  ->  transf(Key,Type)
    :- type(Type).

    rule  transf(Key,Type), enc(Kek2, km+exp)  ->  enc(Key, Kek2+Type)
    :- type(Type).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rule  enc(pdkey, km)  -> sec.
rule  pdkey -> sec.
rule  enc(pan, pdkey) -> sec.

query sec.

